# GreedyKernelMethods

This package implements multiple greedy algorithms for kernel interpolation, i.e. for approximating a function f via the approximation `g(x) = Σ_{1≤i≤l} α_i κ(z_i,·)` satisfying `g(z_i)=f(z_i)` for every `1≤j≤l` where `κ` is a user-chosen kernel function and `l` a fixed integer.


## Supported algorithms

We use the following notations:
- z₁,…,z_m the chosen interpolation nodes;
- Km=[κ(z_i,z_j)]_{1≤i,j≤m} the associated kernel matrix;
- Pm the orthogonal projection on span((ϕ(z_i))_{1≤i≤m}).

The package includes three iterative algorithms to select the interpolation points `z_1,…,z_m`:
- Greedy maximization of det(K_m) 
	(also known as P-greedy approximation)		
- Iterative minimization of ‖(I-Pm)f‖∞
	(also known as orthogonal matching pursuit, f-greedy approximation)		
- Greedy minimization of ‖(I-Pm)f‖
	(also known as f/P-greedy approximation)

In this implementation, the atom selection step at each iteration is *approximated using an exhaustive search over the data* (and not performed e.g. by gradient descent).

## Usage

The package essentially provides the two folloing functions:
- `greedy_interpolation`. 
    This is the main function, and the first argument should be of type `MaximizeDeterminant`, `MaximizeResidual` or `GreedyResidualNormMinimization` in order to select the desired selection criterion.
    It will return a pair containing the selected nodes and associated weights.
- `max_determinant`.

Please refer directly to the documentation of these functions.

## Note on complexity

These algorithms have complexity O(nl(l+d)) assuming a kernel evaluation in O(d), even if the residual and optimal weights are not computed. 

Note that in the setting where the function to interpolate is an empirical kernel mean embedding:
- Evaluating f costs O(n²)	(and is required both for greedy minimization of the residual's norm and OMP)
- Maximizing the determinant can be done without providing f_X and thus avoiding the O(n²) (the optimal weigts then need to be computed separately, but this is O(nl+l³) anyway).

